# -*- coding: utf-8 -*-

import base64
import codecs
import crypt
from hashlib import md5, sha1


# From: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/325204
def apache_md5crypt(password, salt, magic='$apr1$'):
    """
    Calculates the Apache-style MD5 hash of a password
    """
    password = password.encode('utf-8')
    salt = salt.encode('utf-8')
    magic = magic.encode('utf-8')

    m = md5()
    m.update(password + magic + salt)

    mixin = md5(password + salt + password).digest()
    for i in range(0, len(password)):
        m.update(mixin[i % 16])

    i = len(password)
    while i:
        if i & 1:
            m.update('\x00')
        else:
            m.update(password[0])
        i >>= 1

    final = m.digest()

    for i in range(1000):
        m2 = md5()
        if i & 1:
            m2.update(password)
        else:
            m2.update(final)

        if i % 3:
            m2.update(salt)

        if i % 7:
            m2.update(password)

        if i & 1:
            m2.update(final)
        else:
            m2.update(password)

        final = m2.digest()

    itoa64 = './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'

    rearranged = ''
    seq = ((0, 6, 12), (1, 7, 13), (2, 8, 14), (3, 9, 15), (4, 10, 5))
    for a, b, c in seq:
        v = ord(final[a]) << 16 | ord(final[b]) << 8 | ord(final[c])
        for i in range(4):
            rearranged += itoa64[v & 0x3f]
            v >>= 6

    v = ord(final[11])
    for i in range(2):
        rearranged += itoa64[v & 0x3f]
        v >>= 6

    return magic + salt + '$' + rearranged


def check_password(test_password, password):
    if password.startswith('$apr1$'):
        salt = password.strip('$').split('$')[1]
        check = apache_md5crypt(test_password, salt)
    elif password.startswith('{SHA}'):
        _hash = sha1(test_password.encode('utf-8')).digest()
        check = '{SHA}' + base64.b64encode(_hash)
    else:
        check = crypt.crypt(test_password.encode('utf-8'), password)

    return check == password


def read_htpasswd(path):
    f = codecs.open(path, 'r', 'utf-8')
    try:
        users = {}
        for line in f.readlines():
            line = line.strip()
            if not line or line.startswith('#'):
                continue

            try:
                username, password = line.split(':', 1)
            except ValueError:
                continue

            users[username] = password

        return users
    finally:
        f.close()



# -*- coding: utf-8 -*-

import base64
from flask import Response, g, request

try:
    from flask import _app_ctx_stack as stack
except ImportError:
    from flask import _request_ctx_stack as stack

from functools import wraps

from .htpasswd import check_password, read_htpasswd

__all__ = ['HTAuth', 'authenticated']


class HTAuth(object):
    """
    This class controls basic HTTP authentication integration with Flask apps.

    You can either bind the app to an instance::

        app = Flask(__app__)
        auth = HTAuth(app)

    or create the auth object once and bind it later::

        auth = HTAuth()

        def create_app():
            app = Flask(__name__)
            auth.init_app(app)
            return app
    """

    def __init__(self, app=None):
        self._app = app

        if self._app:
            self.init_app(self._app)

    def init_app(self, app):
        """Bind the **app** to HTAuth instance."""
        app.before_request(self._before_request)

    def _before_request(self):
        ctx = stack.top
        if ctx and not hasattr(ctx, 'htauth'):
            if 'HTAUTH_HTPASSWD_PATH' in ctx.app.config:
                _users = read_htpasswd(ctx.app.config['HTAUTH_HTPASSWD_PATH'])
                _realm = ctx.app.config.get('HTAUTH_REALM', 'Protected Area').\
                    encode('utf-8')

                settings = {
                    'users': _users,
                    'realm': _realm
                }

                ctx.htauth = settings


def _unauthorized_response():
    ctx = stack.top
    if ctx and hasattr(ctx, 'htauth'):
        rsp = Response(
            response='Unauthorized',
            status=401,
            headers={
                'WWW-Authenticate': 'Basic realm="%s"' % ctx.htauth['realm']
            },
            mimetype='text/plain',
            content_type='text/plain;charset=utf-8'
        )

        return rsp

    return None


def authenticated(viewfunc):
    """Decorate **viewfunc** with this decorator to require HTTP auth on the
    view."""
    @wraps(viewfunc)
    def wrapper(*args, **kwargs):
        ctx = stack.top
        if ctx and hasattr(ctx, 'htauth'):
            auth_header = request.headers.get('Authorization', None)
            if not auth_header:
                return _unauthorized_response()

            if not auth_header.startswith('Basic '):
                raise RuntimeError('Flask-HTAuth supports only Basic auth.')

            auth_header = auth_header.replace('Basic ', '')
            auth_header = base64.b64decode(auth_header)
            username, password = auth_header.split(':', 1)

            if not username in ctx.htauth['users']:
                return _unauthorized_response()

            if not check_password(password, ctx.htauth['users'][username]):
                return _unauthorized_response()

            g.htauth_user = username

        return viewfunc(*args, **kwargs)

    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012 by Tomasz Wójcik <tomek@bthlabs.pl>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import codecs
from setuptools import setup

version = '0.1.2'

desc_file = codecs.open('README.rst', 'r', 'utf-8')
long_description = desc_file.read()
desc_file.close()

setup(
    name="Flask-HTAuth",
    version=version,
    packages=['flask_htauth'],
    test_suite='nose.collector',
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Flask>=0.6',
    ],
    tests_require=[
        'nose',
    ],
    author=u'Tomasz Wójcik'.encode('utf-8'),
    author_email='tomek@bthlabs.pl',
    maintainer=u'Tomasz Wójcik'.encode('utf-8'),
    maintainer_email='tomek@bthlabs.pl',
    url='http://tomekwojcik.github.com/flask-htauth/',
    download_url='https://github.com/tomekwojcik/flask-htauth/tarball/v%s' % version,
    description='Easy to integrate basic HTTP authentication for Flask apps',
    long_description=long_description,
    license='https://github.com/tomekwojcik/flask-htauth/blob/master/LICENSE',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        "Topic :: Software Development :: Libraries :: Python Modules"
    ]
)